// $Header: Frag.h,v 1.3 99/07/21 16:51:23 vern Exp $
//
// Copyright (c) 1996-1999
//      The Regents of the University of California.  All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that: (1) source code distributions
// retain the above copyright notice and this paragraph in its entirety, (2)
// distributions including binary code include the above copyright notice and
// this paragraph in its entirety in the documentation or other materials
// provided with the distribution, and (3) all advertising materials mentioning
// features or use of this software display the following acknowledgement:
// ``This product includes software developed by the University of California,
// Lawrence Berkeley Laboratory and its contributors.'' Neither the name of
// the University nor the names of its contributors may be used to endorse
// or promote products derived from this software without specific prior
// written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

#ifndef frag_h
#define frag_h

#include "util.h"
#include "Net.h"
#include "Reassem.h"
#include "Timer.h"

class HashKey;
class NetSessions;

class FragReassembler;
class FragTimer;

typedef void (FragReassembler::*frag_timer_func)(double t);

class FragReassembler : public Reassembler {
public:
	FragReassembler(NetSessions* s, const struct ip* ip, const u_char* pkt,
			uint32 frag_field, HashKey* k,double t);
	~FragReassembler();

	void AddFragment(double t, const struct ip* ip, const u_char* pkt, uint32 frag_field);

	const struct ip* ReassembledPkt()	{ return reassembled_pkt; }
	HashKey* Key() const	{ return key; }
	void DeleteTimer();

protected:
	void BlockInserted(double t, DataBlock* start_block);
	void Overlap(const u_char* b1, const u_char* b2, int n);

	// Timer implementation for ip defrag.
        void AddTimer(frag_timer_func timer, double delta_t, int do_expire);
	void ExpireTimer(double t);

	friend FragTimer;
        void RemoveTimer(Timer* t)      { timers.remove(t); }

	// Cancel all associated timers.  If the parameter is true,
        // then the call is itself being made from a timer.
        void CancelTimers(int callee_is_timer);

	struct ip* proto_hdr;
	struct ip* reassembled_pkt;
	int proto_hdr_len;
	NetSessions* s;
	int frag_size;	// size of fully reassembled fragment
	HashKey* key;

	timer_list timers;
	int is_active;
};

class FragTimer : public Timer {
public:
        FragTimer(FragReassembler* arg_f, frag_timer_func arg_timer,
                        double arg_t, int arg_do_expire) : Timer(arg_t)
                { Init(arg_f, arg_timer, arg_do_expire); }
        ~FragTimer();

        void Dispatch(double t, int is_expire);

protected:
        void Init(FragReassembler* f, frag_timer_func timer, int do_expire);

        FragReassembler* f;
        frag_timer_func timer;
        int do_expire;
};

#define ADD_FRAG_TIMER(timer, t, do_expire) \
        AddTimer(frag_timer_func(timer), (t), (do_expire))


#endif
