// $Header: /home/ram/u0/vern/bro/RCS/NetVar.cc,v 1.47 2001/09/05 04:22:13 vern Exp $
//
// Copyright (c) 1996-1999
//      The Regents of the University of California.  All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that: (1) source code distributions
// retain the above copyright notice and this paragraph in its entirety, (2)
// distributions including binary code include the above copyright notice and
// this paragraph in its entirety in the documentation or other materials
// provided with the distribution, and (3) all advertising materials mentioning
// features or use of this software display the following acknowledgement:
// ``This product includes software developed by the University of California,
// Lawrence Berkeley Laboratory and its contributors.'' Neither the name of
// the University nor the names of its contributors may be used to endorse
// or promote products derived from this software without specific prior
// written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
// WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

#include "Var.h"
#include "NetVar.h"

RecordType* conn_id;
RecordType* endpoint;
RecordType* endpoint_stats;
RecordType* connection_type;
RecordType* icmp_conn;
RecordType* icmp_context;

Func* new_connection;
Func* new_connection_contents;
Func* connection_attempt;
Func* connection_established;
Func* partial_connection;
Func* connection_partial_close;
Func* connection_finished;
Func* connection_half_finished;
Func* connection_rejected;
Func* connection_reset;
Func* connection_pending;

Func* packet_contents;

Func* udp_request;
Func* udp_reply;
Func* udp_session_done;

Func* icmp_sent;
Func* icmp_echo_request;
Func* icmp_echo_reply;
Func* icmp_unreachable;

RecordType* net_stats;
Func* net_stats_update;
Func* conn_stats;

int watchdog_interval;
double heartbeat_interval;

int max_timer_expires;

Func* conn_weird;
Func* conn_weird_addl;
Func* flow_weird;
Func* net_weird;
Func* rexmit_inconsistency;
Func* ack_above_hole;

double frag_timeout;

int partial_connection_ok;
int tcp_SYN_ack_ok;

double tcp_SYN_timeout;
double tcp_session_timer;
double tcp_connection_linger;
double tcp_attempt_delay;
double tcp_close_delay;
double tcp_reset_delay;
double tcp_partial_close_delay;

double non_analyzed_lifetime;

int tcp_storm_thresh;
double tcp_storm_interarrival_thresh;

double dns_session_timeout;
double ntp_session_timeout;
double rpc_timeout;

Func* inconsistent_option;
Func* bad_option;
Func* bad_option_termination;

Func* finger_request;
Func* finger_reply;

Func* ident_request;
Func* ident_reply;
Func* ident_error;

ListVal* skip_authentication;
ListVal* direct_login_prompts;
ListVal* login_prompts;
ListVal* login_non_failure_msgs;
ListVal* login_failure_msgs;
ListVal* login_success_msgs;
ListVal* login_timeouts;

Func* login_failure;
Func* login_success;
Func* login_input_line;
Func* login_output_line;
Func* login_confused;
Func* login_confused_text;
Func* login_terminal;
Func* login_display;
Func* excessive_line;

Func* authentication_accepted;
Func* authentication_rejected;
Func* authentication_skipped;

Func* activating_encryption;

Func* ftp_request;
Func* ftp_reply;

Func* http_request;
Func* http_reply;
Func* http_stats;
RecordType* http_stats_rec;

int pm_request;

Func* pm_request_null;
Func* pm_request_set;
Func* pm_request_unset;
Func* pm_request_getport;
Func* pm_request_dump;
Func* pm_request_callit;
Func* pm_attempt_null;
Func* pm_attempt_set;
Func* pm_attempt_unset;
Func* pm_attempt_getport;
Func* pm_attempt_dump;
Func* pm_attempt_callit;
Func* pm_bad_port;

RecordType* pm_mapping;
TableType* pm_mappings;
RecordType* pm_port_request;
RecordType* pm_callit_request;

RecordType* ntp_msg;
Func* ntp_message;

RecordType* dns_msg;
RecordType* dns_answer;
RecordType* dns_soa;

Func* dns_request;
Func* dns_full_request;

Func* dns_A_reply;
Func* dns_NS_reply;
Func* dns_CNAME_reply;
Func* dns_PTR_reply;
Func* dns_SOA_reply;
Func* dns_WKS_reply;
Func* dns_HINFO_reply;
Func* dns_MX_reply;
Func* dns_TXT_reply;
Func* dns_SRV_reply;

Func* dns_EDNS0;

double stp_delta;
double stp_idle_min;

Func* stp_create_endp;
Func* stp_resume_endp;
Func* stp_correlate_pair;
Func* stp_remove_pair;
Func* stp_remove_endp;

double interconn_min_interarrival;
double interconn_max_interarrival;
int interconn_max_keystroke_pkt_size;
int interconn_default_pkt_size;
double interconn_stat_period;
double interconn_stat_backoff;
RecordType* interconn_endp_stats;

Func* interconn_remove_conn;
Func* interconn_stats;

double backdoor_stat_period;
double backdoor_stat_backoff;

RecordType* backdoor_endp_stats;
Func* backdoor_stats;
Func* backdoor_remove_conn;
Func* rlogin_signature_found;
Func* ssh_signature_found;
Func* telnet_signature_found;
Func* root_backdoor_signature_found;
Func* ftp_signature_found;
Func* napster_signature_found;
Func* gnutella_signature_found;

void init_net_var()
	{
	conn_id = internal_type("conn_id")->AsRecordType();
	endpoint = internal_type("endpoint")->AsRecordType();
	endpoint_stats = internal_type("endpoint_stats")->AsRecordType();
	connection_type = internal_type("connection")->AsRecordType();
	icmp_conn = internal_type("icmp_conn")->AsRecordType();
	icmp_context = internal_type("icmp_context")->AsRecordType();

	new_connection = internal_func("new_connection");
	new_connection_contents = internal_func("new_connection_contents");
	connection_attempt = internal_func("connection_attempt");
	connection_established = internal_func("connection_established");
	partial_connection = internal_func("partial_connection");
	connection_partial_close = internal_func("connection_partial_close");
	connection_finished = internal_func("connection_finished");
	connection_half_finished = internal_func("connection_half_finished");
	connection_rejected = internal_func("connection_rejected");
	connection_reset = internal_func("connection_reset");
	connection_pending = internal_func("connection_pending");

	packet_contents = internal_func("packet_contents");

	udp_request = internal_func("udp_request");
	udp_reply = internal_func("udp_reply");
	udp_session_done = internal_func("udp_session_done");

	icmp_sent = internal_func("icmp_sent");
	icmp_echo_request = internal_func("icmp_echo_request");
	icmp_echo_reply = internal_func("icmp_echo_reply");
	icmp_unreachable = internal_func("icmp_unreachable");

	conn_weird = internal_func("conn_weird");
	conn_weird_addl = internal_func("conn_weird_addl");
	flow_weird = internal_func("flow_weird");
	net_weird = internal_func("net_weird");
	rexmit_inconsistency = internal_func("rexmit_inconsistency");
	ack_above_hole = internal_func("ack_above_hole");

	frag_timeout = opt_internal_double("frag_timeout");

	partial_connection_ok = opt_internal_int("partial_connection_ok");
	tcp_SYN_ack_ok = opt_internal_int("tcp_SYN_ack_ok");

	tcp_SYN_timeout = opt_internal_double("tcp_SYN_timeout");
	tcp_session_timer = opt_internal_double("tcp_session_timer");
	tcp_connection_linger = opt_internal_double("tcp_connection_linger");
	tcp_attempt_delay = opt_internal_double("tcp_attempt_delay");
	tcp_close_delay = opt_internal_double("tcp_close_delay");
	tcp_reset_delay = opt_internal_double("tcp_reset_delay");
	tcp_partial_close_delay = opt_internal_double("tcp_partial_close_delay");

	non_analyzed_lifetime = opt_internal_double("non_analyzed_lifetime");

	tcp_storm_thresh = opt_internal_int("tcp_storm_thresh");
	tcp_storm_interarrival_thresh =
		opt_internal_double("tcp_storm_interarrival_thresh");

	dns_session_timeout = opt_internal_double("dns_session_timeout");
	ntp_session_timeout = opt_internal_double("ntp_session_timeout");
	rpc_timeout = opt_internal_double("rpc_timeout");

	inconsistent_option = internal_func("inconsistent_option");
	bad_option = internal_func("bad_option");
	bad_option_termination = internal_func("bad_option_termination");

	net_stats = internal_type("net_stats")->AsRecordType();
	net_stats_update = internal_func("net_stats_update");
	conn_stats = internal_func("conn_stats");

	watchdog_interval = int(opt_internal_double("watchdog_interval"));
	heartbeat_interval = opt_internal_double("heartbeat_interval");

	max_timer_expires = opt_internal_int("max_timer_expires");

	finger_request = internal_func("finger_request");
	finger_reply = internal_func("finger_reply");

	ident_request = internal_func("ident_request");
	ident_reply = internal_func("ident_reply");
	ident_error = internal_func("ident_error");

	skip_authentication = internal_list_val("skip_authentication");
	direct_login_prompts = internal_list_val("direct_login_prompts");
	login_prompts = internal_list_val("login_prompts");
	login_non_failure_msgs = internal_list_val("login_non_failure_msgs");
	login_failure_msgs = internal_list_val("login_failure_msgs");
	login_success_msgs = internal_list_val("login_success_msgs");
	login_timeouts = internal_list_val("login_timeouts");

	login_failure = internal_func("login_failure");
	login_success = internal_func("login_success");
	login_input_line = internal_func("login_input_line");
	login_output_line = internal_func("login_output_line");
	login_confused = internal_func("login_confused");
	login_confused_text = internal_func("login_confused_text");
	login_terminal = internal_func("login_terminal");
	login_display = internal_func("login_display");
	excessive_line = internal_func("excessive_line");

	authentication_accepted = internal_func("authentication_accepted");
	authentication_rejected = internal_func("authentication_rejected");
	authentication_skipped = internal_func("authentication_skipped");

	activating_encryption = internal_func("activating_encryption");

	ftp_request = internal_func("ftp_request");
	ftp_reply = internal_func("ftp_reply");

	http_request = internal_func("http_request");
	http_reply = internal_func("http_reply");
	http_stats = internal_func("http_stats");
	http_stats_rec = internal_type("http_stats_rec")->AsRecordType();

	pm_request_null = internal_func("pm_request_null");
	pm_request_set = internal_func("pm_request_set");
	pm_request_unset = internal_func("pm_request_unset");
	pm_request_getport = internal_func("pm_request_getport");
	pm_request_dump = internal_func("pm_request_dump");
	pm_request_callit = internal_func("pm_request_callit");

	pm_attempt_null = internal_func("pm_attempt_null");
	pm_attempt_set = internal_func("pm_attempt_set");
	pm_attempt_unset = internal_func("pm_attempt_unset");
	pm_attempt_getport = internal_func("pm_attempt_getport");
	pm_attempt_dump = internal_func("pm_attempt_dump");
	pm_attempt_callit = internal_func("pm_attempt_callit");

	pm_bad_port = internal_func("pm_bad_port");

	pm_request = pm_request_null || pm_request_set ||
		pm_request_unset || pm_request_getport ||
		pm_request_dump || pm_request_callit ||
		pm_attempt_null || pm_attempt_set ||
		pm_attempt_unset || pm_attempt_getport ||
		pm_attempt_dump || pm_attempt_callit ||
		pm_bad_port;

	pm_mapping = internal_type("pm_mapping")->AsRecordType();
	pm_mappings = internal_type("pm_mappings")->AsTableType();
	pm_port_request = internal_type("pm_port_request")->AsRecordType();
	pm_callit_request = internal_type("pm_callit_request")->AsRecordType();

	ntp_msg = internal_type("ntp_msg")->AsRecordType();
	ntp_message = internal_func("ntp_message");

	dns_msg = internal_type("dns_msg")->AsRecordType();
	dns_answer = internal_type("dns_answer")->AsRecordType();
	dns_soa = internal_type("dns_soa")->AsRecordType();

	dns_request = internal_func("dns_request");
	dns_full_request = internal_func("dns_full_request");

	dns_A_reply = internal_func("dns_A_reply");
	dns_NS_reply = internal_func("dns_NS_reply");
	dns_CNAME_reply = internal_func("dns_CNAME_reply");
	dns_PTR_reply = internal_func("dns_PTR_reply");
	dns_SOA_reply = internal_func("dns_SOA_reply");
	dns_WKS_reply = internal_func("dns_WKS_reply");
	dns_HINFO_reply = internal_func("dns_HINFO_reply");
	dns_MX_reply = internal_func("dns_MX_reply");
	dns_TXT_reply = internal_func("dns_TXT_reply");
	dns_SRV_reply = internal_func("dns_SRV_reply");

	dns_EDNS0 = internal_func("dns_EDNS0");

	stp_delta = opt_internal_double("stp_delta");
	stp_idle_min = opt_internal_double("stp_idle_min");

	stp_create_endp = internal_func("stp_create_endp");
	stp_resume_endp = internal_func("stp_resume_endp");
	stp_correlate_pair = internal_func("stp_correlate_pair");
	stp_remove_pair = internal_func("stp_remove_pair");
	stp_remove_endp = internal_func("stp_remove_endp");

	interconn_min_interarrival = opt_internal_double("interconn_min_interarrival");
	interconn_max_interarrival = opt_internal_double("interconn_max_interarrival");
	interconn_max_keystroke_pkt_size = opt_internal_int("interconn_max_keystroke_pkt_size");
	interconn_default_pkt_size = opt_internal_int("interconn_default_pkt_size");
	interconn_stat_period = opt_internal_double("interconn_stat_period");
	interconn_stat_backoff = opt_internal_double("interconn_stat_backoff");

	interconn_endp_stats = internal_type("interconn_endp_stats")->AsRecordType();

	interconn_remove_conn = internal_func("interconn_remove_conn");
	interconn_stats = internal_func("interconn_stats");

	backdoor_stat_period = opt_internal_double("backdoor_stat_period");
	backdoor_stat_backoff = opt_internal_double("backdoor_stat_backoff");
	backdoor_endp_stats = internal_type("backdoor_endp_stats")->AsRecordType();
	backdoor_stats = internal_func("backdoor_stats");
	backdoor_remove_conn = internal_func("backdoor_remove_conn");

	rlogin_signature_found = internal_func("rlogin_signature_found");
	ssh_signature_found = internal_func("ssh_signature_found");
	telnet_signature_found = internal_func("telnet_signature_found");
	root_backdoor_signature_found = internal_func("root_backdoor_signature_found");
	ftp_signature_found = internal_func("ftp_signature_found");
	napster_signature_found = internal_func("napster_signature_found");
	gnutella_signature_found = internal_func("gnutella_signature_found");
	}
